!RAM_dir = $60		; Current direction of player
!RAM_ani = $61		; Current animation frame of player
!RAM_timer = $62	; Timer for charged shot
!RAM_buttons = $63	; Previous buttons

!RAM_powerup = $1F2B	; Poweup flags
; Format: x-----cs
; c: Allow charged shots
; s: Allow A for running
; x: Must be 0
; -: Unused

!DEF_charged = $40	; How many frames the player needs to hold the shoot button for a charged shot

!charged_snd = $29
!charged_bnk = $1DFC

!xspeed = 16	; Horizontal speed (not running)
!yspeed = 16	; Vertical speed (not running)

!xspeed2 = 32	; Horizontal speed (running)
!yspeed2 = 32	; Vertical speed (running)

!sprXSpeed = 20	; Horizontal projectile speed
!sprYSpeed = 20	; Vertical projectile speed

!slow = $3		; Animation slowdown

!spr = $00		; Sprite to shoot
!custom = 1		; Custom sprite? (1 = yes, 0 = no)

Tilemap:
;down
	db $00,$01,$02,$01
	db $00,$00,$00,$00

;up
	db $04,$05,$06,$05
	db $00,$00,$00,$00

;left
	db $07,$08,$09,$08
	db $01,$01,$01,$01

;right
	db $07,$08,$09,$08
	db $00,$00,$00,$00

;downRight
	db $07,$08,$09,$08
	db $00,$00,$00,$00

;downLeft
	db $07,$08,$09,$08
	db $01,$01,$01,$01

;upRight
	db $07,$08,$09,$08
	db $00,$00,$00,$00

;upLeft
	db $07,$08,$09,$08
	db $01,$01,$01,$01


; Speed tables. Use defines above to change.
Speed:
.x	db $00,!xspeed,-!xspeed,$00,$00,!xspeed,-!xspeed,$00,$00,!xspeed,-!xspeed,$00,$00,!xspeed,-!xspeed,$00
	db $00,!xspeed2,-!xspeed2,$00,$00,!xspeed2,-!xspeed2,$00,$00,!xspeed2,-!xspeed2,$00,$00,!xspeed2,-!xspeed2,$00
.y	db $00,$00,$00,$00,!yspeed,!yspeed,!yspeed,!yspeed,-!yspeed,-!yspeed,-!yspeed,-!yspeed,$00,$00,$00,$00
	db $00,$00,$00,$00,!yspeed2,!yspeed2,!yspeed2,!yspeed2,-!yspeed2,-!yspeed2,-!yspeed2,-!yspeed2,$00,$00,$00,$00

; Direction table. Do not change.
direction:
	db $FF,$03,$02,$00,$00,$04,$05,$00,$01,$06,$07,$01,$00,$03,$02,$00

main:
	LDA $9D
	BNE +
	JSR UpdatePos
+	LDA !RAM_ani
	LSR #!slow
	STA $00
	LDA !RAM_dir
	ASL #3
	CLC : ADC $00
	TAX
	CLC : ADC #$04
	TAY
	LDA Tilemap,x
	STA $13E0
	LDA Tilemap,y
	STA $76
	RTL

UpdatePos:
	LDA #$FF	; \ Make player 'climb' so he isn't affected by gravity.
	STA $74		; /
	LDA $15 
	AND #$0F
	BEQ +	; If no buttons are pressed return since we won't have to recalculate position. Also don't increment animation timer
	TAY
	LDA $17		; If L or R is pressed don't update direction the player faces.
	AND #$30
	BNE ++
	LDA direction,y
	STA !RAM_dir
++	LDA !RAM_powerup	; \ If player has the powerup flag set...
	ORA $17			; | ...and is pressing A...
	AND #$81		; |
	CMP #$81		; |
	BNE ++			; |
	TYA				; | ...use second speed table
	CLC : ADC #$10	; |
	TAY				; /
++	LDA !RAM_ani				; \ Update animation frame
	INC : AND #$FF>>(6-!slow)	; |
	STA !RAM_ani				; /
	LDA Speed_x,y	; \ Set X speed of player
	STA $7B			; /
	LDA Speed_y,y	; \ Set Y speed of player
	STA $7D			; /
+	LDA $16			; \ Disable A/B button
	AND #$7F		; |
	STA $16			; /
	LDA !RAM_buttons
	AND #$40
	BEQ +
	LDA $15
	AND #$40
	BNE ++
	JSR Shoot
	BCS +
	LDA !RAM_timer
	CMP #!DEF_charged
	BCC +
	LDA !RAM_powerup
	AND #$02
	BEQ +++
	LDA #$01
	STA $C2,x
	BRA +
++	LDA !RAM_powerup
	AND #$02
	BEQ +++
	INC !RAM_timer
	LDA !RAM_timer
	CMP #!DEF_charged
	BCC +++
	BNE ++
	LDA #!charged_snd
	STA !charged_bnk
	BRA +++
++	DEC !RAM_timer
	BRA +++
+	STZ !RAM_timer
+++	LDA $15
	STA !RAM_buttons
	RTS

; Projectile speed table. Use above defines to change.
Spr:
.xS db $00,$00,-!sprXSpeed,!sprXSpeed,!sprXSpeed,-!sprXSpeed,!sprXSpeed,-!sprXSpeed
.yS	db !sprYSpeed,-!sprYSpeed,$00,$00,!sprYSpeed,!sprYSpeed,-!sprYSpeed,-!sprYSpeed

Shoot:
	LDA #!spr
	if !custom == 1
		SEC
	else
		CLC
	endif
	JSL TrySpawn
	BCS .return
	TAX
	LDA $94
	STA $E4,x
	LDA $95
	STA $14E0,x
	LDA $96
	CLC : ADC #$10
	STA $D8,x
	LDA $97
	ADC #$00
	STA $14D4,x
	LDY !RAM_dir
	LDA Spr_xS,y
	STA $B6,x
	LDA Spr_yS,y
	STA $AA,x
	CLC
.return
	RTS

; Totally not taken from gps
TrySpawn:
	PHX
	XBA
	LDX #$0B
	-
		LDA $14C8,x
		BEQ +
			DEX
		BPL -
		SEC
		BRA no_slot
	+
	XBA
	STA $9E,x
	JSL $07F7D2	
	BCC +
		LDA $9E,x
		STA $7FAB9E,x
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
	+
	LDA #$01
	STA $14C8,x
	CLC
no_slot:
	TXA
	PLX
	RTL
	
	
		

macro displayRAM(ram, pos)
	PHA
	LDA <ram>
	AND #$0F
	STA $0EFA
	LSR #4
	STA $0EF9
	PLA
endmacro